/////////////////////////////////////////////////////////////
// Flash Plugin and Player
// Copyright (C) 1998 Olivier Debon
// 
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
// 
///////////////////////////////////////////////////////////////
#ifndef _PROGRAM_H_
#define _PROGRAM_H_

struct Frame {
	char *label;
	Control *controls;	// Controls for this frame
};

enum MovieStatus {
	MoviePaused,
	MoviePlay
};

struct FlashMovie;

struct Program {
        DisplayList	*dl;

	Frame		*frames;	// Array
	long		 nbFrames;	// Number of valid frames
	long  		 currentFrame;
	long  		 loadingFrame;
	long  		 totalFrames;	// Total expected number of frames
	long  		 nextFrame;
	int		 movieWait;	// If true freeze movie until next loaded frame
	MovieStatus 	 movieStatus;
	Sound		*currentSound;
	long		 settings;
        FlashMovie      *movie;
	long		 render;	// True if needed to be rendered

	Program(FlashMovie *movie,long n);
	~Program();

	void	 rewindMovie();
	void	 pauseMovie();
	void	 continueMovie();
	void	 nextStepMovie();
	void	 gotoFrame(GraphicDevice *gd, long f);

	long	 processMovie(GraphicDevice *, SoundMixer *);
	long	 nestedMovie(GraphicDevice *, SoundMixer *, Matrix *, Cxform *);
	long	 runFrame(GraphicDevice *, SoundMixer *, long f, long action=1);
	long	 handleEvent(GraphicDevice *, SoundMixer *, FlashEvent *);
	long	 doAction(GraphicDevice *gd, ActionRecord *action, SoundMixer *);
	void	 setCurrentFrameLabel(char *label);
	void	 advanceFrame();
	void	 addControlInCurrentFrame(Control *ctrl);
	void	 setGetUrlMethod( void (*)(char *, char *, void *), void *);
	void	 modifySettings(long flags);
	long	 searchFrame(GraphicDevice *gd, char *, char *);
	void	 validateLoadingFrame();
	long	 getCurrentFrame();
	void	 setCurrentFrame(long);

	Frame	*getFrames();
	long	 getNbFrames();

	DisplayList *getDisplayList();

#ifdef DUMP
	void	 dump(BitStream *bs);
static  void	 dumpActions(BitStream *bs, ActionRecord *actions);
#endif
};

DisplayListEntry *findFocus(DisplayList *dl);
void setFlashTimer(struct timeval *tv, int time_ms);
int checkFlashTimer(struct timeval *tv);

void loadNewSwf(FlashMovie *movie, char *url, int level);

void computeBBox(FlashMovie *movie, Rect *rect, DisplayListEntry *e);

long processMovie(FlashMovie *movie);

#endif /* _PROGRAM_H_ */
